@@ -6,6 +6,7 @@ import os
 import time
 import sys
 import struct
+import socket
 import logging
 import threading
 import traceback
@@ -228,6 +229,13 @@ def uuid32():
                             os.getpid()))
 
 
+def dqn2int(mask):
+    '''
+    IPv4 dotted quad notation to int mask conversion
+    '''
+    return bin(struct.unpack('>L', socket.inet_aton(mask))[0]).count('1')
+
+
 def hexdump(payload, length=0):
     '''
     Represent byte string as hex -- for debug purposes
