@@ -2,15 +2,15 @@
 IPDB module
 ===========
 
-Experimental IP database module. Provides high-level interface
-to the IPRoute functionality.
-
 Basically, IPDB is a transactional database, containing records,
 representing network stack objects. Any change in the database
 is not reflected immediately in OS (unless you ask for that
 explicitly), but waits until commit() is called.
 
-Quick start::
+quickstart
+----------
+
+Simple tutorial::
 
     from pyroute2 import IPDB
     # several IPDB instances are supported within on process
@@ -25,9 +25,176 @@ Quick start::
 
     # basic routing support
     ip.routes.add({'dst': 'default', 'gateway': '10.0.0.1'}).commit()
+
+IPDB uses IPRoute as a transport, and monitors all broadcast
+netlink messages from the kernel, thus keeping the database
+up-to-date in an asynchronous manner. IPDB inherits `dict`
+class, and has two keys::
+
+    >>> from pyroute2 import IPDB
+    >>> ip = IPDB()
+    >>> ip.by_name.keys()
+    ['bond0', 'lo', 'em1', 'wlan0', 'dummy0', 'virbr0-nic', 'virbr0']
+    >>> ip.by_index.keys()
+    [32, 1, 2, 3, 4, 5, 8]
+    >>> ip.interfaces.keys()
+    [32,
+     1,
+     2,
+     3,
+     4,
+     5,
+     8,
+     'lo',
+     'em1',
+     'wlan0',
+     'bond0',
+     'dummy0',
+     'virbr0-nic',
+     'virbr0']
+    >>> ip.interfaces['em1']['address']
+    'f0:de:f1:93:94:0d'
+    >>> ip.interfaces['em1']['ipaddr']
+    [('10.34.131.210', 23),
+     ('2620:52:0:2282:f2de:f1ff:fe93:940d', 64),
+     ('fe80::f2de:f1ff:fe93:940d', 64)]
+    >>>
+
+One can address objects in IPDB not only with dict notation, but
+with dot notation also:
+
+    >>> ip.interfaces.em1.address
+    'f0:de:f1:93:94:0d'
+    >>> ip.interfaces.em1.ipaddr
+    [('10.34.131.210', 23),
+     ('2620:52:0:2282:f2de:f1ff:fe93:940d', 64),
+     ('fe80::f2de:f1ff:fe93:940d', 64)]
+    ```
+
+It is up to you, which way to choose. The former, being more flexible,
+is better for developers, the latter, the shorter form -- for system
+administrators.
+
+
+The library has also IPDB module. It is a database synchronized with
+the kernel, containing some of the information. It can be used also
+to set up IP settings in a transactional manner:
+
+    >>> from pyroute2 import IPDB
+    >>> from pprint import pprint
+    >>> ip = IPDB()
+    >>> pprint(ip.by_name.keys())
+    ['bond0',
+     'lo',
+     'vnet0',
+     'em1',
+     'wlan0',
+     'macvtap0',
+     'dummy0',
+     'virbr0-nic',
+     'virbr0']
+    >>> ip.interfaces.lo
+    {'promiscuity': 0,
+     'operstate': 'UNKNOWN',
+     'qdisc': 'noqueue',
+     'group': 0,
+     'family': 0,
+     'index': 1,
+     'linkmode': 0,
+     'ipaddr': [('127.0.0.1', 8), ('::1', 128)],
+     'mtu': 65536,
+     'broadcast': '00:00:00:00:00:00',
+     'num_rx_queues': 1,
+     'txqlen': 0,
+     'ifi_type': 772,
+     'address': '00:00:00:00:00:00',
+     'flags': 65609,
+     'ifname': 'lo',
+     'num_tx_queues': 1,
+     'ports': [],
+     'change': 0}
+    >>>
+
+transaction modes
+-----------------
+IPDB has several operating modes:
+
+ * 'direct' -- any change goes immediately to the OS level
+ * 'implicit' (default) -- the first change starts an implicit
+   transaction, that have to be committed
+ * 'explicit' -- you have to begin() a transaction prior to
+   make any change
+ * 'snapshot' -- no changes will go to the OS in any case
+
+The default is to use implicit transaction. This behaviour can
+be changed in the future, so use 'mode' argument when creating
+IPDB instances.
+
+The sample session with explicit transactions::
+
+    In [1]: from pyroute2 import IPDB
+    In [2]: ip = IPDB(mode='explicit')
+    In [3]: ifdb = ip.interfaces
+    In [4]: ifdb.tap0.begin()
+        Out[3]: UUID('7a637a44-8935-4395-b5e7-0ce40d31d937')
+    In [5]: ifdb.tap0.up()
+    In [6]: ifdb.tap0.address = '00:11:22:33:44:55'
+    In [7]: ifdb.tap0.add_ip('10.0.0.1', 24)
+    In [8]: ifdb.tap0.add_ip('10.0.0.2', 24)
+    In [9]: ifdb.tap0.review()
+        Out[8]:
+        {'+ipaddr': set([('10.0.0.2', 24), ('10.0.0.1', 24)]),
+         '-ipaddr': set([]),
+         'address': '00:11:22:33:44:55',
+         'flags': 4099}
+    In [10]: ifdb.tap0.commit()
+
+
+Note, that you can `review()` the `last()` transaction, and
+`commit()` or `drop()` it. Also, multiple `self._transactions`
+are supported, use uuid returned by `begin()` to identify them.
+
+Actually, the form like 'ip.tap0.address' is an eye-candy. The
+IPDB objects are dictionaries, so you can write the code above
+as that::
+
+    ip.interfaces['tap0'].down()
+    ip.interfaces['tap0']['address'] = '00:11:22:33:44:55'
+    ...
+
+context managers
+----------------
+
+Also, interface objects in transactional mode can operate as
+context managers::
+
+    with ip.interfaces.tap0 as i:
+        i.address = '00:11:22:33:44:55'
+        i.ifname = 'vpn'
+        i.add_ip('10.0.0.1', 24)
+        i.add_ip('10.0.0.1', 24)
+
+On exit, the context manager will authomatically `commit()` the
+transaction.
+
+interface creation
+------------------
+
+IPDB can also create interfaces::
+
+    with ip.create(kind='bridge', ifname='control') as i:
+        i.add_port(ip.interfaces.eth1)
+        i.add_port(ip.interfaces.eth2)
+        i.add_ip('10.0.0.1/24')  # the same as i.add_ip('10.0.0.1', 24)
+
+Right now IPDB supports creation of `dummy`, `bond`, `bridge`
+and `vlan` interfaces. VLAN creation requires also `link` and
+`vlan_id` parameters, see example scripts.
+
 '''
 import os
 import uuid
+import socket
 import threading
 try:
     from Queue import Empty
@@ -37,6 +204,7 @@ except ImportError:
 from socket import AF_UNSPEC
 from socket import AF_INET
 from socket import AF_INET6
+from pyroute2.common import dqn2int
 from pyroute2.common import Dotkeys
 from pyroute2.netlink import NetlinkError
 from pyroute2.netlink.ipdb import compat
@@ -50,13 +218,17 @@ tc_fields = [tcmsg.nla2name(i[0]) for i in tcmsg.nla_map]
 
 # How long should we wait on EACH commit() checkpoint: for ipaddr,
 # ports etc. That's not total commit() timeout.
-_SYNC_TIMEOUT = 3
+_SYNC_TIMEOUT = 5
 
 _FAIL_COMMIT = 0b00000001
 _FAIL_ROLLBACK = 0b00000010
 _FAIL_MASK = 0b11111111
 
 
+class CommitException(Exception):
+    pass
+
+
 def clear_fail_bit(bit):
     global _FAIL_MASK
     _FAIL_MASK &= ~(_FAIL_MASK & bit)
@@ -69,10 +241,24 @@ def set_fail_bit(bit):
 
 def get_interface_type(name):
     '''
-    Utility function to get interface type
+    Utility function to get interface type.
+
+    Unfortunately, we can not rely on RTNL or even ioctl().
+    RHEL doesn't support interface type in RTNL and doesn't
+    provide extended (private) interface flags via ioctl().
+
+    Args:
+        * name (str): interface name
+
+    Returns:
+        * False -- sysfs info unavailable
+        * None -- type not known
+        * str -- interface type:
+            * 'bond'
+            * 'bridge'
     '''
-    # we can not even rely on ioctl(), since RHEL does not support
-    # extended (private) interface flags :(((
+    # FIXME: support all interface types? Right now it is
+    # not needed
     try:
         ifattrs = os.listdir('/sys/class/net/%s/' % (name))
     except OSError as e:
@@ -91,12 +277,21 @@ def get_interface_type(name):
 
 def get_addr_nla(msg):
     '''
+    Utility function to get NLA, containing the interface
+    address.
+
     Incosistency in Linux IP addressing scheme is that
     IPv4 uses IFA_LOCAL to store interface's ip address,
     and IPv6 uses for the same IFA_ADDRESS.
 
     IPv4 sets IFA_ADDRESS to == IFA_LOCAL or to a
     tunneling endpoint.
+
+    Args:
+        * msg (nlmsg): RTM\_.*ADDR message
+
+    Returns:
+        * nla (nla): IFA_LOCAL for IPv4 and IFA_ADDRESS for IPv6
     '''
     nla = None
     if msg['family'] == AF_INET:
@@ -107,6 +302,16 @@ def get_addr_nla(msg):
 
 
 class LinkedSet(set):
+    '''
+    Utility class, used by `Interface` to track ip addresses
+    and ports. Called "linked" as it automatically updates all
+    instances, linked with it.
+
+    Target filter is a function, that returns `True` if a set
+    member should be counted in target checks (target methods
+    see below), or `False` if it should be ignored.
+    '''
+    target_filter = lambda self, x: True
 
     def __init__(self, *argv, **kwarg):
         set.__init__(self, *argv, **kwarg)
@@ -117,6 +322,14 @@ class LinkedSet(set):
         self.links = []
 
     def set_target(self, value):
+        '''
+        Set target state for the object and clear the target
+        event. Once the target is reached, the event will be
+        set, see also: `check_target()`
+
+        Args:
+            * value (set): the target state to compare with
+        '''
         with self.lock:
             if value is None:
                 self._ct = None
@@ -126,13 +339,31 @@ class LinkedSet(set):
                 self.target.clear()
 
     def check_target(self):
+        '''
+        Check the target state and set the target event in the
+        case the state is reached. Called from mutators, `add()`
+        and `remove()`
+        '''
         with self.lock:
             if self._ct is not None:
-                if self == self._ct:
+                if set(filter(self.target_filter, self)) == \
+                        set(filter(self.target_filter, self._ct)):
                     self._ct = None
                     self.target.set()
 
     def add(self, key, raw=None):
+        '''
+        Add an item to the set and all connected instances,
+        check the target state.
+
+        Args:
+            * key: any hashable object
+            * raw (optional): raw representation of the object
+
+        Raw representation is not required. It can be used, e.g.,
+        to store RTM_NEWADDR RTNL messages along with
+        human-readable ip addr representation.
+        '''
         with self.lock:
             if key not in self:
                 self.raw[key] = raw
@@ -142,6 +373,10 @@ class LinkedSet(set):
             self.check_target()
 
     def remove(self, key, raw=None):
+        '''
+        Remove an item from the set and all connected instances,
+        check the target state.
+        '''
         with self.lock:
             set.remove(self, key)
             for link in self.links:
@@ -150,6 +385,10 @@ class LinkedSet(set):
             self.check_target()
 
     def connect(self, link):
+        '''
+        Connect a LinkedSet instance to this one. Connected
+        sets will be updated together with this instance.
+        '''
         assert isinstance(link, LinkedSet)
         self.links.append(link)
 
@@ -157,6 +396,15 @@ class LinkedSet(set):
         return repr(list(self))
 
 
+class IPaddrSet(LinkedSet):
+    '''
+    LinkedSet child class with different target filter. The
+    filter ignores link local IPv6 addresses when sets and checks
+    the target.
+    '''
+    target_filter = lambda self, x: not ((x[0][:4] == 'fe80') and (x[1] == 64))
+
+
 class State(object):
 
     def __init__(self, lock=None):
@@ -242,6 +490,10 @@ class IPRequest(dict):
 
 
 class IPRouteRequest(IPRequest):
+    '''
+    Utility class, that converts human-readable dictionary
+    into RTNL route request.
+    '''
 
     def __setitem__(self, key, value):
         if (key == 'dst') and (value != 'default'):
@@ -261,6 +513,10 @@ class IPRouteRequest(IPRequest):
 
 
 class IPLinkRequest(IPRequest):
+    '''
+    Utility class, that converts human-readable dictionary
+    into RTNL link request.
+    '''
 
     def __setitem__(self, key, value):
         if key == 'kind':
@@ -285,10 +541,13 @@ class IPLinkRequest(IPRequest):
 
 class Transactional(Dotkeys):
     '''
-    An utility class that implements common transactional logic.
+    Utility class that implements common transactional logic.
     '''
+    _fields_cmp = {}
+
     def __init__(self, ipdb, mode=None):
         self.nl = ipdb.nl
+        self.nlmsg = None
         self.uid = uuid.uuid4()
         self.ipdb = ipdb
         self.last_error = None
@@ -296,6 +555,7 @@ class Transactional(Dotkeys):
         self._fields = []
         self._tids = []
         self._transactions = {}
+        self._targets = {}
         self._mode = mode or ipdb.mode
         self._write_lock = threading.RLock()
         self._direct_state = State(self._write_lock)
@@ -356,8 +616,6 @@ class Transactional(Dotkeys):
         return res.__repr__()
 
     def __sub__(self, vs):
-        '''
-        '''
         res = self.__class__(ipdb=self.ipdb, mode='snapshot')
         with self._direct_state:
             # simple keys
@@ -423,7 +681,13 @@ class Transactional(Dotkeys):
         if not direct:
             transaction = self.last()
             transaction[key] = value
+            transaction._targets[key] = threading.Event()
         else:
+            for tn in self._transactions.values():
+                if key in tn._targets:
+                    if self._fields_cmp.\
+                            get(key, lambda x, y: x == y)(value, tn[key]):
+                        tn._targets[key].set()
             Dotkeys.__setitem__(self, key, value)
 
     @update
@@ -453,8 +717,6 @@ class Transactional(Dotkeys):
 
 
 class TrafficControl(Transactional):
-    '''
-    '''
     def __init__(self, ipdb, mode=None):
         Transactional.__init__(self, ipdb, mode)
         self._fields = tc_fields
@@ -495,6 +757,8 @@ class Interface(Transactional):
     exception will be raised. Failed transaction review
     will be attached to the exception.
     '''
+    _fields_cmp = {'flags': lambda x, y: x & y}
+
     def __init__(self, ipdb, mode=None):
         '''
         Parameters:
@@ -514,20 +778,19 @@ class Interface(Transactional):
         self.egress = None
         self._exists = False
         self._flicker = False
+        self._virtual_fields = ('removal', 'flicker', 'state')
         self._fields = [ifinfmsg.nla2name(i[0]) for i in ifinfmsg.nla_map]
         self._fields.append('flags')
         self._fields.append('mask')
         self._fields.append('change')
-        self._fields.append('state')
-        self._fields.append('removal')
-        self._fields.append('flicker')
+        self._fields.extend(self._virtual_fields)
         self._load_event = threading.Event()
         self._linked_sets.add('ipaddr')
         self._linked_sets.add('ports')
         # 8<-----------------------------------
         # local setup: direct state is required
         with self._direct_state:
-            self['ipaddr'] = LinkedSet()
+            self['ipaddr'] = IPaddrSet()
             self['ports'] = LinkedSet()
             for i in self._fields:
                 self[i] = None
@@ -556,8 +819,9 @@ class Interface(Transactional):
         '''
         with self._direct_state:
             self._exists = True
-            self._flicker = False
-            self.update(dev)
+            self.nlmsg = dev
+            for (name, value) in dev.items():
+                self[name] = value
             for (name, value) in dev['attrs']:
                 norm = ifinfmsg.nla2name(name)
                 self[norm] = value
@@ -598,7 +862,10 @@ class Interface(Transactional):
         # split mask
         if mask is None:
             ip, mask = ip.split('/')
-            mask = int(mask, 0)
+            if mask.find('.') > -1:
+                mask = dqn2int(mask)
+            else:
+                mask = int(mask, 0)
         # FIXME: make it more generic
         # skip IPv6 link-local addresses
         if ip[:4] == 'fe80' and mask == 64:
@@ -617,7 +884,10 @@ class Interface(Transactional):
         '''
         if mask is None:
             ip, mask = ip.split('/')
-            mask = int(mask, 0)
+            if mask.find('.') > -1:
+                mask = dqn2int(mask)
+            else:
+                mask = int(mask, 0)
         if not direct:
             transaction = self.last()
             if (ip, mask) in transaction['ipaddr']:
@@ -680,219 +950,211 @@ class Interface(Transactional):
         Commit transaction. In the case of exception all
         changes applied during commit will be reverted.
         '''
-        error = None
-        added = None
-        removed = None
-        if tid:
-            transaction = self._transactions[tid]
-        else:
-            transaction = transaction or self.last()
+        with self.ipdb.exclusive:
+            error = None
+            added = None
+            removed = None
+            if tid:
+                transaction = self._transactions[tid]
+            else:
+                transaction = transaction or self.last()
+
+            # if the interface does not exist, create it first ;)
+            if not self._exists:
+                request = IPLinkRequest(self)
+                self.ipdb._links_event.clear()
+
+                # create watchdog
+                wd = self.ipdb.watchdog(ifname=self['ifname'])
 
-        # if the interface does not exist, create it first ;)
-        if not self._exists:
-            request = IPLinkRequest(self)
-            self.ipdb._links_event.clear()
-            try:
-                # 8<----------------------------------------------------
-                # ACHTUNG: hack for old platforms
-                if request.get('address', None) == '00:00:00:00:00:00':
-                    del request['address']
-                    del request['broadcast']
-                # 8<----------------------------------------------------
                 try:
-                    compat.fix_create_link(self.nl, request)
-                except NetlinkError as x:
-                    # Operation not supported
-                    if x.code == 95 and request.get('index', 0) != 0:
-                        # ACHTUNG: hack for old platforms
-                        request = IPLinkRequest({'ifname': self['ifname'],
-                                                 'kind': self['kind'],
-                                                 'index': 0})
+                    # 8<----------------------------------------------------
+                    # ACHTUNG: hack for old platforms
+                    if request.get('address', None) == '00:00:00:00:00:00':
+                        del request['address']
+                        del request['broadcast']
+                    # 8<----------------------------------------------------
+                    try:
                         compat.fix_create_link(self.nl, request)
-                    else:
-                        raise
-            except Exception:
-                # on failure, invalidate the interface and detach it
-                # from the parent
-                # 1. drop the IPRoute() link
-                self.nl = None
-                # 2. clean up ipdb
-                self.ipdb.detach(self['index'])
-                self.ipdb.detach(self['ifname'])
-                # 3. invalidate the interface
-                with self._direct_state:
-                    for i in tuple(self.keys()):
-                        del self[i]
-                # 4. the rest
-                self._mode = 'invalid'
-                # raise the exception
-                raise
-
-            # all is OK till now, so continue
-            # we do not know what to load, so load everything
-            self.ipdb.wait_interface(ifname=self['ifname'])
-
-        # now we have our index and IP set and all other stuff
-        snapshot = self.pick()
-
-        try:
-            removed = snapshot - transaction
-            added = transaction - snapshot
-
-            # 8<---------------------------------------------
-            # IP address changes
-            self['ipaddr'].set_target(transaction['ipaddr'])
-
-            for i in removed['ipaddr']:
-                # When you remove a primary IP addr, all subnetwork
-                # can be removed. In this case you will fail, but
-                # it is OK, no need to roll back
-                try:
-                    self.nl.addr('delete', self['index'], i[0], i[1])
-                except NetlinkError as x:
-                    # bypass only errno 99, 'Cannot assign address'
-                    if x.code != 99:
-                        raise
-
-            for i in added['ipaddr']:
-                self.nl.addr('add', self['index'], i[0], i[1])
-
-            if removed['ipaddr'] or added['ipaddr']:
-                self['ipaddr'].target.wait(_SYNC_TIMEOUT)
-                assert self['ipaddr'].target.is_set()
-
-            # 8<---------------------------------------------
-            # Interface slaves
-            self['ports'].set_target(transaction['ports'])
-
-            for i in removed['ports']:
-                # detach the port
-                compat.fix_del_port(self.nl, self, self.ipdb.interfaces[i])
-
-            for i in added['ports']:
-                # enslave the port
-                compat.fix_add_port(self.nl, self, self.ipdb.interfaces[i])
-
-            if removed['ports'] or added['ports']:
-                self.nl.get_links(*(removed['ports'] | added['ports']))
-                self['ports'].target.wait(_SYNC_TIMEOUT)
-                assert self['ports'].target.is_set()
-
-            # 8<---------------------------------------------
-            # Interface changes
-            request = IPLinkRequest()
-            for key in added:
-                if key in self._fields:
-                    request[key] = added[key]
-
-            # apply changes only if there is something to apply
-            if any([request[item] is not None for item in request]):
-                self.nl.link('set', index=self['index'], **request)
-
-            # 8<---------------------------------------------
-            # Interface removal
-            if added.get('removal') or added.get('flicker'):
-                self._load_event.clear()
-                if added.get('flicker'):
-                    self.ipdb.flicker.add(self['index'])
-                compat.fix_del_link(self.nl, self)
-                self._load_event.wait(_SYNC_TIMEOUT)
-                assert self._load_event.is_set()
-                if added.get('flicker'):
-                    self.ipdb.flicker.remove(self['index'])
-                    self._exists = False
-                    self._flicker = True
-                if added.get('removal'):
+                    except NetlinkError as x:
+                        # Operation not supported
+                        if x.code == 95 and request.get('index', 0) != 0:
+                            # ACHTUNG: hack for old platforms
+                            request = IPLinkRequest({'ifname': self['ifname'],
+                                                     'kind': self['kind'],
+                                                     'index': 0})
+                            compat.fix_create_link(self.nl, request)
+                        else:
+                            raise
+                except Exception:
+                    # on failure, invalidate the interface and detach it
+                    # from the parent
+                    # 1. drop the IPRoute() link
+                    self.nl = None
+                    # 2. clean up ipdb
+                    self.ipdb.detach(self['index'])
+                    self.ipdb.detach(self['ifname'])
+                    # 3. invalidate the interface
+                    with self._direct_state:
+                        for i in tuple(self.keys()):
+                            del self[i]
+                    # 4. the rest
                     self._mode = 'invalid'
-                self.drop()
-                return self
-            # 8<---------------------------------------------
+                    # raise the exception
+                    raise
 
-            if rollback:
-                assert _FAIL_ROLLBACK & _FAIL_MASK
-            else:
+                # all is OK till now, so continue
+                # we do not know what to load, so load everything
+                wd.wait()
 
-                # 8<-----------------------------------------
-                # Iterate callback chain
-                for cb in self._callbacks:
-                    # An exception will rollback the transaction
-                    cb(snapshot, transaction)
-                # 8<-----------------------------------------
+            # now we have our index and IP set and all other stuff
+            snapshot = self.pick()
 
-                assert _FAIL_COMMIT & _FAIL_MASK
+            try:
+                removed = snapshot - transaction
+                added = transaction - snapshot
 
-        except Exception as e:
-            # something went wrong: roll the transaction back
-            if not rollback:
-                self.reload()
-                # 8<-----------------------------------------
-                # That's a hack, but we have to use it, since
-                # OS response can be not so fast
-                # * inject added IPs directly into self
-                # * wipe removed IPs from the interface data
-                #
-                # This info will be used to properly roll back
-                # changes.
-                #
-                # Still, there is a possibility that the
-                # rollback will run even before IP addrs will
-                # be assigned. But we can not cope with that.
-                with self._direct_state:
-                    if removed:
-                        for i in removed['ipaddr']:
-                            try:
-                                self['ipaddr'].remove(i)
-                            except KeyError:
-                                pass
-                    if added:
-                        for i in added['ipaddr']:
-                            self['ipaddr'].add(i)
-                # 8<-----------------------------------------
-                ret = self.commit(transaction=snapshot, rollback=True)
-                # if some error was returned by the internal
-                # closure, substitute the initial one
-                if isinstance(ret, Exception):
-                    error = ret
+                # 8<---------------------------------------------
+                # IP address changes
+                self['ipaddr'].set_target(transaction['ipaddr'])
+
+                for i in removed['ipaddr']:
+                    # When you remove a primary IP addr, all subnetwork
+                    # can be removed. In this case you will fail, but
+                    # it is OK, no need to roll back
+                    try:
+                        self.nl.addr('delete', self['index'], i[0], i[1])
+                    except NetlinkError as x:
+                        # bypass only errno 99, 'Cannot assign address'
+                        if x.code != 99:
+                            raise
+                    except socket.error as x:
+                        # bypass illegal IP requests
+                        if not x.args[0].startswith('illegal IP'):
+                            raise
+
+                for i in added['ipaddr']:
+                    self.nl.addr('add', self['index'], i[0], i[1])
+
+                if removed['ipaddr'] or added['ipaddr']:
+                    self['ipaddr'].target.wait(_SYNC_TIMEOUT)
+                    if not self['ipaddr'].target.is_set():
+                        raise CommitException('ipaddr target is not set')
+
+                # 8<---------------------------------------------
+                # Interface slaves
+                self['ports'].set_target(transaction['ports'])
+
+                for i in removed['ports']:
+                    # detach the port
+                    compat.fix_del_port(self.nl, self, self.ipdb.interfaces[i])
+
+                for i in added['ports']:
+                    # enslave the port
+                    compat.fix_add_port(self.nl, self, self.ipdb.interfaces[i])
+
+                if removed['ports'] or added['ports']:
+                    self.nl.get_links(*(removed['ports'] | added['ports']))
+                    self['ports'].target.wait(_SYNC_TIMEOUT)
+                    if not self['ports'].target.is_set():
+                        raise CommitException('ports target is not set')
+
+                # 8<---------------------------------------------
+                # Interface changes
+                request = IPLinkRequest()
+                for key in added:
+                    if key in self._fields:
+                        request[key] = added[key]
+
+                # apply changes only if there is something to apply
+                if any([request[item] is not None for item in request]):
+                    self.nl.link('set', index=self['index'], **request)
+                    self.reload()
+
+                # wait for targets
+                for key, target in transaction._targets.items():
+                    if key not in self._virtual_fields:
+                        target.wait(_SYNC_TIMEOUT)
+                        if not target.is_set():
+                            raise CommitException('target %s is not set' % key)
+
+                # 8<---------------------------------------------
+                # Interface removal
+                if added.get('removal') or added.get('flicker'):
+                    wd = self.ipdb.watchdog(action='RTM_DELLINK',
+                                            ifname=self['ifname'])
+                    if added.get('flicker'):
+                        self._flicker = True
+                    compat.fix_del_link(self.nl, self)
+                    wd.wait()
+                    if added.get('flicker'):
+                        self._exists = False
+                    if added.get('removal'):
+                        self._mode = 'invalid'
+                    self.drop()
+                    return self
+                # 8<---------------------------------------------
+
+                if rollback:
+                    assert _FAIL_ROLLBACK & _FAIL_MASK
                 else:
-                    error = e
-            elif isinstance(e, NetlinkError) and getattr(e, 'code', 0) == 1:
-                # It is <Operation not permitted>, catched in
-                # rollback. So return it -- see ~5 lines above
-                return e
-            else:
-                # somethig went wrong during automatic rollback.
-                # that's the worst case, but it is still possible,
-                # since we have no locks on OS level.
+
+                    # 8<-----------------------------------------
+                    # Iterate callback chain
+                    for cb in self._callbacks:
+                        # An exception will rollback the transaction
+                        cb(snapshot, transaction)
+                    # 8<-----------------------------------------
+
+                    assert _FAIL_COMMIT & _FAIL_MASK
+
+            except Exception as e:
+                # something went wrong: roll the transaction back
+                if not rollback:
+                    ret = self.commit(transaction=snapshot, rollback=True)
+                    # if some error was returned by the internal
+                    # closure, substitute the initial one
+                    if isinstance(ret, Exception):
+                        error = ret
+                    else:
+                        error = e
+                elif isinstance(e, NetlinkError) and \
+                        getattr(e, 'code', 0) == 1:
+                    # It is <Operation not permitted>, catched in
+                    # rollback. So return it -- see ~5 lines above
+                    return e
+                else:
+                    # somethig went wrong during automatic rollback.
+                    # that's the worst case, but it is still possible,
+                    # since we have no locks on OS level.
+                    self.drop()
+                    self['ipaddr'].set_target(None)
+                    self['ports'].set_target(None)
+                    # reload all the database -- it can take a long time,
+                    # but it is required since we have no idea, what is
+                    # the result of the failure
+                    #
+                    # ACHTUNG: database reload is asynchronous, so after
+                    # getting RuntimeError() from commit(), take a seat
+                    # and rest for a while. It is an extremal case, it
+                    # should not became at all, and there is no sync.
+                    self.nl.get_links()
+                    self.nl.get_addr()
+                    x = RuntimeError()
+                    x.cause = e
+                    raise x
+
+            # if it is not a rollback turn
+            if not rollback:
+                # drop last transaction in any case
                 self.drop()
-                self['ipaddr'].set_target(None)
-                self['ports'].set_target(None)
-                # reload all the database -- it can take a long time,
-                # but it is required since we have no idea, what is
-                # the result of the failure
-                #
-                # ACHTUNG: database reload is asynchronous, so after
-                # getting RuntimeError() from commit(), take a seat
-                # and rest for a while. It is an extremal case, it
-                # should not became at all, and there is no sync.
-                self.nl.get_links()
-                self.nl.get_addr()
-                x = RuntimeError()
-                x.cause = e
-                raise x
 
-        # if it is not a rollback turn
-        if not rollback:
-            # drop last transaction in any case
-            self.drop()
-            # re-load interface information
-            self.reload()
+            # raise exception for failed transaction
+            if error is not None:
+                error.transaction = transaction
+                raise error
 
-        # raise exception for failed transaction
-        if error is not None:
-            error.transaction = transaction
-            raise error
-
-        return self
+            return self
 
     def up(self):
         '''
@@ -1114,6 +1376,32 @@ class RoutingTables(dict):
         return self.remove(key)
 
 
+class Watchdog(object):
+    def __init__(self, ipdb, action, kwarg):
+        self.event = threading.Event()
+        self.ipdb = ipdb
+
+        def cb(ipdb, msg, _action):
+            if _action != action:
+                return
+
+            for key in kwarg:
+                if (msg.get(key, None) != kwarg[key]) and \
+                        (msg.get_attr(msg.name2nla(key)) != kwarg[key]):
+                    return
+            self.event.set()
+        self.cb = cb
+        # register callback prior to other things
+        self.ipdb.register_callback(self.cb)
+
+    def wait(self, timeout=_SYNC_TIMEOUT):
+        self.event.wait(timeout=timeout)
+        self.cancel()
+
+    def cancel(self):
+        self.ipdb.unregister_callback(self.cb)
+
+
 class IPDB(object):
     '''
     The class that maintains information about network setup
@@ -1154,7 +1442,6 @@ class IPDB(object):
         self.by_index = Dotkeys()
 
         # caches
-        self.flicker = set()
         self.ipaddr = {}
         self.neighbors = {}
 
@@ -1287,28 +1574,31 @@ class IPDB(object):
 
         FIXME: this should be documented.
         '''
-        # check for existing interface
-        if ((ifname in self.interfaces) and
-                (self.interfaces[ifname]._flicker)):
-            device = self.interfaces[ifname]
-        else:
-            device = \
-                self.by_name[ifname] = \
-                self.interfaces[ifname] = \
-                self.iclass(ipdb=self, mode='snapshot')
-            device.update(kwarg)
-            if isinstance(kwarg.get('link', None), Interface):
-                device['link'] = kwarg['link']['index']
-            device['kind'] = kind
-            device['index'] = kwarg.get('index', 0)
-            device['ifname'] = ifname
-            device._mode = self.mode
-        device.begin()
-        return device
+        with self.exclusive:
+            # check for existing interface
+            if ((ifname in self.interfaces) and
+                    (self.interfaces[ifname]._flicker)):
+                device = self.interfaces[ifname]
+                device._flicker = False
+            else:
+                device = \
+                    self.by_name[ifname] = \
+                    self.interfaces[ifname] = \
+                    self.iclass(ipdb=self, mode='snapshot')
+                device.update(kwarg)
+                if isinstance(kwarg.get('link', None), Interface):
+                    device['link'] = kwarg['link']['index']
+                device['kind'] = kind
+                device['index'] = kwarg.get('index', 0)
+                device['ifname'] = ifname
+                device._mode = self.mode
+            device.begin()
+            return device
 
     def device_del(self, msg):
         # check for flicker devices
-        if (msg.get('index', None) in self.flicker):
+        if (msg.get('index', None) in self.interfaces) and \
+                self.interfaces[msg['index']]._flicker:
             self.interfaces[msg['index']].sync()
             return
         try:
@@ -1370,7 +1660,7 @@ class IPDB(object):
 
         if index not in self.ipaddr:
             # for interfaces, created by IPDB
-            self.ipaddr[index] = LinkedSet()
+            self.ipaddr[index] = IPaddrSet()
 
         device.load(msg)
         if not skip_slaves:
@@ -1380,31 +1670,8 @@ class IPDB(object):
         if item in self.interfaces:
             del self.interfaces[item]
 
-    def wait_interface(self, action='RTM_NEWLINK', **kwarg):
-        event = threading.Event()
-
-        def cb(self, msg, _action):
-            if _action != action:
-                return
-            try:
-                port = self.interfaces[msg['index']]
-            except KeyError:
-                return
-
-            for key in kwarg:
-                if port.get(key, None) != kwarg[key]:
-                    return
-            event.set()
-
-        # register callback prior to other things
-        self.register_callback(cb)
-        # inspect existing interfaces, as if they were created
-        for index in self.by_index:
-            cb(self, self.by_index[index], 'RTM_NEWLINK')
-        # ok, wait the event
-        event.wait()
-        # unregister callback
-        self.unregister_callback(cb)
+    def watchdog(self, action='RTM_NEWLINK', **kwarg):
+        return Watchdog(self, action, kwarg)
 
     def update_routes(self, routes):
         for msg in routes:
